#!/usr/bin/env python3
"""
Buckia - Multi-platform, multi-backend storage bucket synchronization

This setup.py file is provided for backward compatibility.
For modern Python packaging, see pyproject.toml.
"""

import setuptools

# Redirect to setuptools.setup with pyproject.toml configuration
if __name__ == "__main__":
    setuptools.setup()
